@@ -21,12 +21,6 @@ my $null_io = do { open my $io, "<", \""; $io };
 
 use Net::Server::SIG qw(register_sig);
 
-# Override Net::Server's HUP handling - just restart all the workers and that's about it
-sub sig_hup {
-    my $self = shift;
-    $self->hup_children;
-}
-
 sub run {
     my($self, $app, $options) = @_;
 
@@ -43,9 +37,6 @@ sub run {
     if (! exists $options->{keepalive}) {
         $options->{keepalive} = 1;
     }
-    if (! exists $options->{keepalive_timeout}) {
-        $options->{keepalive_timeout} = 1;
-    }
 
     my($host, $port, $proto);
     for my $listen (@{$options->{listen} || [ "$options->{host}:$options->{port}" ]}) {
@@ -70,7 +61,6 @@ sub run {
         proto                      => $proto,
         serialize                  => 'flock',
         log_level                  => DEBUG ? 4 : 2,
-        ($options->{error_log} ? ( log_file => $options->{error_log} ) : () ),
         min_servers                => $options->{min_servers}       || $workers,
         min_spare_servers          => $options->{min_spare_servers} || $workers - 1,
         max_spare_servers          => $options->{max_spare_servers} || $workers - 1,
@@ -79,7 +69,7 @@ sub run {
         user                       => $options->{user}              || $>,
         group                      => $options->{group}             || $),
         listen                     => $options->{backlog}           || 1024,
-        check_for_waiting          => 1,
+        leave_children_open_on_hup => 1, # XXX conigurable?
         no_client_stdout           => 1,
         %extra
     );
@@ -101,37 +91,12 @@ sub pre_loop_hook {
     register_sig(
         TTIN => sub { $self->{server}->{$_}++ for qw( min_servers max_servers ) },
         TTOU => sub { $self->{server}->{$_}-- for qw( min_servers max_servers ) },
-        QUIT => sub { $self->server_close(1) },
     );
 }
 
-sub server_close {
-    my($self, $quit) = @_;
-
-    if ($quit) {
-        $self->log(2, $self->log_time . " Received QUIT. Running a graceful shutdown\n");
-        $self->{server}->{$_} = 0 for qw( min_servers max_servers );
-        $self->hup_children;
-        while (1) {
-            Net::Server::SIG::check_sigs();
-            $self->coordinate_children;
-            last if !keys %{$self->{server}{children}};
-            sleep 1;
-        }
-        $self->log(2, $self->log_time . " Worker processes cleaned up\n");
-    }
-
-    $self->SUPER::server_close();
-}
-
 sub run_parent {
     my $self = shift;
     $0 = "starman master " . join(" ", @{$self->{options}{argv} || []});
-    local *Net::Server::PreFork::register_sig = sub {
-        my %args = @_;
-        delete $args{QUIT};
-        Net::Server::SIG::register_sig(%args);
-    };
     $self->SUPER::run_parent(@_);
 }
 
@@ -188,7 +153,6 @@ sub process_request {
             'psgi.multiprocess' => Plack::Util::TRUE,
             'psgix.io'          => $conn,
             'psgix.input.buffered' => Plack::Util::TRUE,
-            'psgix.harakiri' => Plack::Util::TRUE,
         };
 
         # Parse headers
@@ -266,7 +230,7 @@ sub process_request {
             if ( $self->{client}->{inputbuf} ) {
                 if ( $self->{client}->{inputbuf} =~ /^(?:GET|HEAD)/ ) {
                     if ( DEBUG ) {
-                        warn "Pipelined GET/HEAD request in input buffer: "
+                        warn "Pipelined GET/HEAD request in input buffer: " 
                             . dump( $self->{client}->{inputbuf} ) . "\n";
                     }
 
@@ -287,7 +251,7 @@ sub process_request {
             DEBUG && warn "[$$] Waiting on previous connection for keep-alive request...\n";
 
             my $sel = IO::Select->new($conn);
-            last unless $sel->can_read($self->{options}->{keepalive_timeout});
+            last unless $sel->can_read(1);
         }
     }
 
@@ -429,11 +393,6 @@ sub _prepare_env {
 sub _finalize_response {
     my($self, $env, $res) = @_;
 
-    if ($env->{'psgix.harakiri.commit'}) {
-        $self->{client}->{keepalive} = 0;
-        $self->{client}->{harakiri} = 1;
-    }
-
     my $protocol = $env->{SERVER_PROTOCOL};
     my $status   = $res->[0];
     my $message  = status_message($status);
@@ -443,10 +402,7 @@ sub _finalize_response {
 
     # Switch on Transfer-Encoding: chunked if we don't know Content-Length.
     my $chunked;
-    my $headers = $res->[1];
-    for (my $i = 0; $i < @$headers; $i += 2) {
-        my $k = $headers->[$i];
-        my $v = $headers->[$i + 1];
+    while (my($k, $v) = splice @{$res->[1]}, 0, 2) {
         next if $k eq 'Connection';
         push @headers, "$k: $v";
         $headers{lc $k} = $v;
@@ -495,7 +451,6 @@ sub _finalize_response {
             my $buffer = $_[0];
             if ($chunked) {
                 my $len = length $buffer;
-                return unless $len;
                 $buffer = sprintf( "%x", $len ) . $CRLF . $buffer . $CRLF;
             }
             syswrite $conn, $buffer;
@@ -509,7 +464,6 @@ sub _finalize_response {
                 my $buffer = $_[0];
                 if ($chunked) {
                     my $len = length $buffer;
-                    return unless $len;
                     $buffer = sprintf( "%x", $len ) . $CRLF . $buffer . $CRLF;
                 }
                 syswrite $conn, $buffer;
@@ -521,11 +475,4 @@ sub _finalize_response {
     }
 }
 
-sub post_client_connection_hook {
-    my $self = shift;
-    if ($self->{client}->{harakiri}) {
-        exit;
-    }
-}
-
 1;
